using System.Text;

using Microsoft.CodeAnalysis;
using Microsoft.CodeAnalysis.Text;

namespace NSeguin.Dev.Web.HeroIcons.SourceGenerator;

[Generator]
public class HeroIconsSourceGenerator : IIncrementalGenerator
{
    private const string PathBase = "wwwroot/lib/heroicons/24/solid";

    public void Initialize(IncrementalGeneratorInitializationContext context)
    {
        IncrementalValuesProvider<AdditionalText> files = context.AdditionalTextsProvider.Where(
            t => t.Path.Contains(PathBase) && t.Path.EndsWith(".svg"));

        context.RegisterSourceOutput(files, GenerateCode);
    }

    private static void GenerateCode(SourceProductionContext context, AdditionalText file)
    {
        HeroIconPropertyTemplate template = new(file.Path);
        string source = template.Code;
        context.AddSource(template.OutputFileName, SourceText.From(source, Encoding.UTF8));
    }

    private class HeroIconPropertyTemplate
    {
        public HeroIconPropertyTemplate(string path)
        {
            Path = path.IndexOf(PathBase, StringComparison.Ordinal) switch
            {
                -1 => throw new ArgumentException("Path must contain the base path", nameof(path)),
                var index => path.Substring(index + PathBase.Length + 1)
            };

            SvgFileName = System.IO.Path.GetFileName(path);
            PropertyName = ToPascalCase(System.IO.Path.GetFileNameWithoutExtension(path));
        }

        public string Path { get; }

        public string SvgFileName { get; }

        public string OutputFileName => $"HeroIcon_{PropertyName}.g.cs";

        public string PropertyName { get; }

        public string Code =>
            $$"""
              // <auto-generated/>
              namespace NSeguin.Dev.Web.Components.Shared.HeroIcons
              {
                public partial class HeroIcon
                {
                    /// <summary>
                    /// Gets the <see cref="HeroIconKind"/> for the <c>{{SvgFileName}}</c> icon.
                    /// </summary>
                    public static readonly HeroIconKind {{PropertyName}} = new HeroIconKind("{{SvgFileName}}");
                }
              }
              """;

        private static string ToPascalCase(string value)
        {
            if (string.IsNullOrEmpty(value))
            {
                return value;
            }

            return string.Join(
                string.Empty,
                value.Split(['-', '_'], StringSplitOptions.RemoveEmptyEntries)
                    .Select(
                        word => word.Length == 1
                            ? word.ToUpperInvariant()
                            : char.ToUpperInvariant(word[0]) + word.Substring(1)));
        }
    }
}
