using System.Diagnostics;

using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Rendering;

namespace NSeguin.Dev.Web.Components.Shared.HeroIcons;

public partial class HeroIcon : ComponentBase
{
    private Task<string>? _loadSvgTask;
    private string? _rawSvg;

    [Parameter]
    public HeroIconKind? Kind { get; set; }

    [Parameter]
    public HeroIconVariant Variant { get; set; }

    private string? RawSvg => _rawSvg;

    protected override void BuildRenderTree(RenderTreeBuilder builder)
    {
        builder.AddMarkupContent(0, RawSvg);
    }

    protected override void OnParametersSet()
    {
        if (Kind is not { } kind)
        {
            throw new InvalidOperationException("Kind cannot be null.");
        }

        HeroIconParameters parameters = new(kind, Variant);
        _loadSvgTask = parameters.GetSvgAsync();
        if (_loadSvgTask.IsCompletedSuccessfully)
        {
            TrySetRawSvg(_loadSvgTask.Result);
        }
        else
        {
            _loadSvgTask.ContinueWith(task => TrySetRawSvg(task.Result));
        }
    }

    protected override async Task OnInitializedAsync()
    {
        if (Kind is null)
        {
            throw new InvalidOperationException("Kind cannot be null.");
        }

        if (RawSvg is null && _loadSvgTask is not null)
        {
            await _loadSvgTask;
        }
    }

    private void TrySetRawSvg(string rawSvg)
    {
        Debug.Assert(_loadSvgTask is not null, "_loadSvgTask is not null");
        if (Interlocked.CompareExchange(ref _rawSvg, _loadSvgTask.Result, null) is null)
        {
            StateHasChanged();
        }
    }
}
